package quota.quota;

import java.util.ArrayList;
import java.util.Collections;

import javax.swing.AbstractListModel;

/**
 * Modèle de liste pour Swing.
 * Les éléments sont triés par ordre croissant (suivant l'ordre défini par
 * la méthode compareTo de l'interface Comparable)
 * @param <T> type d'élément de la liste (doit implémenter Comparable<T>)
 */
public class SortedListModel<T extends Comparable<T>> extends AbstractListModel<T>
{
    private ArrayList<T> sortedList;
    
    /**
     * Constructeur
     */
    public SortedListModel()
    {
        this.sortedList = new ArrayList<T>();
    }
    
    /**
     * ajoute l'élément element à la liste (en tenant compte de l'ordre
     * défini par la méthode compareTo de l'interface Comparable)
     * @param element élément à ajouter
     */
    public void add(T element)
    {
    	int idx = Collections.binarySearch(this.sortedList, element);
    	if ( idx < 0 ) {
    		idx = -idx - 1;
    		this.sortedList.add(idx, element);
        	this.fireIntervalAdded(this, idx, idx+1);
   	}
    }
    
    /**
     * supprime l'élément element de la liste
     * @param element élément à supprimer
     */
    public void remove(T element)
    {
    	int idx = Collections.binarySearch(this.sortedList, element);
    	if ( idx >= 0 ) {
    		this.sortedList.remove(idx);
        	this.fireIntervalRemoved(this, idx, idx+1);
    	}
    }
    
    
    /**
     * retourne le nombre d'éléments de la liste
     * Cette méthode est définie par l'interface ListModel
     * @return nombre d'éléments de la liste
     */
    public int getSize()
    {
    	return this.sortedList.size();
    }
    
    /**
     * retourne l'élément d'indice index
     * Cette méthode est définie par l'interface ListModel
     * @param index indice de l'élément
     * @return élément d'indice index
     */
    public T getElementAt(int index)
    {
        return this.sortedList.get(index);
    }
    
}